/*
	$Id: BCLogData.java,v 1.5 2003/02/11 06:43:48 admin Exp $
	-----
	
    SSTP Bottle Client for MacOS X, using protocol SLPP2.0
    Copyright (C) 2001-2002 phonohawk.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
import java.io.*;
import java.util.*;
import java.util.zip.*;
import java.net.*;

public class BCLogData
{
	int year;
	int month;
	int day;
	boolean flag_dirty;
	volatile boolean locked;
    Vector hash_array; // content : (Hashtable)R[h
    
    public BCLogData(int year,int month,int day)
    {
		this.year = year;
		this.month = month;
		this.day = day;
		this.flag_dirty = false;
		this.locked = false;
		this.hash_array = new Vector();
		
		try {
			File raw = getRawFile();
			File gzipped = getGzippedFile();
			if (gzipped.exists()) {
				load(new GZIPInputStream(new FileInputStream(gzipped)));
			}
			else if (raw.exists()) {
				load(new FileInputStream(raw));
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}
    }
	
	public File getRawFile() {
		return new File(BCLogDataServer.home,
				"Library/BotCli/log/"+year+"/"+(month<=9?"0"+month:""+month)+"/"+(day<=9?"0"+day:""+day)+".log");
	}
	
	public File getGzippedFile() {
		return new File(BCLogDataServer.home,
				"Library/BotCli/log/"+year+"/"+(month<=9?"0"+month:""+month)+"/"+(day<=9?"0"+day:""+day)+".log.gz");
	}
	
	public int getYear() {
		return year;
	}
	
	public int getMonth() {
		return month;
	}
	
	public int getDay() {
		return day;
	}
    
    protected void load(InputStream is)
    {
        // sǂݍHashtable쐬B
		DataInputStream dis = new DataInputStream(is);
        try
        {
		  line_loop:
            while(true)
            {
                String line = dis.readUTF(); // EOFOďIB
                line = line.substring(0,line.length() - 1); // Ō̉sB
                
                Hashtable dict = new Hashtable();
            
                StringTokenizer st = new StringTokenizer(line,",");
                while(st.hasMoreTokens())
                {
                    String token = st.nextToken();
                    
                    // label:data `ɂȂĂ̂ŁAȒOŕB
                    String label = token.substring(0,token.indexOf(':'));
                    String data = token.substring(token.indexOf(':') + 1);
                    
                    // xɏ]ăp[XB
					dict.put(label,new String(URLDecoder.decode(data).getBytes(),"SJIS"));
                }
				
                hash_array.add(dict);
            }
        }
		catch (EOFException e)
		{
			// ok
		}
		catch (IOException e)
		{
			System.err.println("Error in loading log stream...");
		}
		
		try{dis.close();}catch(Exception e){}
    }
    
   	public void commit()
	{
		if (flag_dirty) {
			File raw = getRawFile();
			File gzipped = getGzippedFile();
			
			raw.delete();
			gzipped.delete();
			
			lock();
			try {
				if (BCLogFileCompressor.haveToCompress(year,month,day)) {
					// k
					save(new GZIPOutputStream(new FileOutputStream(gzipped)));
				}
				else {
					// kȂB
					save(new FileOutputStream(raw));
				}
			}
			catch (Exception e) {
				e.printStackTrace();
			}
			unlock();
			
			flag_dirty = false;
			//System.out.println("commited: "+toString());
		}
	}
    
    protected void save(OutputStream os) throws Exception
    {
		// type:[^Cv],script:[XNvg],...[LF]
		// ^Cv"temporary-"Ŏn܂ĂۑȂB
		DataOutputStream dos = new DataOutputStream(os);
		final int n_records = hash_array.size();
		for (int i = 0;i < n_records;i++)
		{
			Hashtable record = (Hashtable)hash_array.elementAt(i);
			Object[] entries = record.entrySet().toArray(); // entries : <Map.Entry>
			StringBuffer buf = new StringBuffer();
			for (int j = 0;j < entries.length;j++)
			{
				Map.Entry entry = (Map.Entry)entries[j];
				
				String key = (String)entry.getKey();
				if (key.startsWith("temporary-")) continue;
				
				String value = (String)entry.getValue();
				
				buf.append((buf.length() > 0 ? "," : "")+key+":"+URLEncoder.encode(new String(value.getBytes("SJIS"))));
			}
			dos.writeUTF(buf.toString()+'\n');
		}
		dos.flush();
		dos.close();
    }
	
	public void add(Hashtable bottle)
	{
		lock();
		
		hash_array.add(bottle);
		flag_dirty = true;
		
		unlock();
	}
	
	public void beDirty()
	{
		flag_dirty = true;
	}
	
	public void lock()
	{
		while (locked) {
			try {
				Thread.sleep(100);
			}
			catch (Exception e) {}
		}
		locked = true;
	}
	
	public void unlock()
	{
		locked = false;
	}
    
    public Vector getArray()
    {
		// ԂꂽVectorMȂAbeDirty()Ă΂Ȃ΂ȂȂB
		// elements(),elementAt()ŃbN̓ǂׂBYޏłB
		return hash_array;
    }
	
	public Enumeration elements()
	{
		// ԂꂽHashtableMȂAbeDirty()Ă΂Ȃ΂ȂȂB
		return hash_array.elements();
	}
	
	public Hashtable elementAt(int i)
	{
		// ԂꂽHashtableMȂAbeDirty()Ă΂Ȃ΂ȂȂB
		return (Hashtable)hash_array.elementAt(i);
	}
	
	public int size()
	{
		return hash_array.size();
	}
	
	public String toString()
	{
		return year+"/"+month+"/"+day;
	}
}
